@@ -25,7 +25,6 @@
 #endif
 
 #include <libxfce4ui/libxfce4ui.h>
-#include <exo/exo.h>
 
 #include <common/panel-private.h>
 #include <common/panel-utils.h>
@@ -122,6 +121,10 @@ panel_utils_show_help (GtkWindow   *parent,
 gboolean
 panel_utils_grab_available (void)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+  /* TODO fix for gtk3 */
+  return TRUE;
+#else
   GdkScreen     *screen;
   GdkWindow     *root;
   GdkGrabStatus  grab_pointer = GDK_GRAB_FROZEN;
@@ -166,6 +169,7 @@ panel_utils_grab_available (void)
     }
 
   return grab_succeed;
+#endif
 }
 
 
@@ -201,3 +205,24 @@ panel_utils_set_atk_info (GtkWidget   *widget,
         atk_object_set_description (object, description);
     }
 }
+
+
+
+static gboolean
+destroy_later (gpointer widget)
+{
+  gtk_widget_destroy (GTK_WIDGET (widget));
+  g_object_unref (G_OBJECT (widget));
+  return FALSE;
+}
+
+
+
+void
+panel_utils_destroy_later (GtkWidget *widget)
+{
+  panel_return_if_fail (GTK_IS_WIDGET (widget));
+
+  g_idle_add_full (G_PRIORITY_HIGH, destroy_later, widget, NULL);
+  g_object_ref_sink (G_OBJECT (widget));
+}
