@@ -1001,15 +1001,29 @@ xfce_panel_plugin_button_press_event (GtkWidget      *widget,
 
 
 
+static gboolean
+xfce_panel_plugin_idle_move (XfcePanelPlugin *plugin)
+{
+  panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN (plugin), FALSE);
+  panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (plugin), FALSE);
+
+  /* move the plugin */
+  xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
+                                          PROVIDER_SIGNAL_MOVE_PLUGIN);
+
+  return FALSE;
+}
+
+
+
 static void
 xfce_panel_plugin_menu_move (XfcePanelPlugin *plugin)
 {
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (plugin));
 
-  /* move the plugin */
-  xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (plugin),
-                                          PROVIDER_SIGNAL_MOVE_PLUGIN);
+  /* wait for the popup to go down */
+  g_idle_add ((GSourceFunc) xfce_panel_plugin_idle_move, plugin);
 }
 
 
@@ -1173,23 +1187,31 @@ xfce_panel_plugin_menu_get (XfcePanelPlugin *plugin)
         {
           /* properties item */
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-          item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
+          item = gtk_image_menu_item_new_with_mnemonic (_("_Properties"));
 G_GNUC_END_IGNORE_DEPRECATIONS
           g_signal_connect_swapped (G_OBJECT (item), "activate",
               G_CALLBACK (xfce_panel_plugin_show_configure), plugin);
           g_object_set_qdata (G_OBJECT (menu), item_properties, item);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+          image = gtk_image_new_from_icon_name ("gtk-properties", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
           if (PANEL_HAS_FLAG (plugin->priv->flags, PLUGIN_FLAG_SHOW_CONFIGURE))
             gtk_widget_show (item);
 
           /* about item */
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-          item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
+          item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
 G_GNUC_END_IGNORE_DEPRECATIONS
           g_signal_connect_swapped (G_OBJECT (item), "activate",
               G_CALLBACK (xfce_panel_plugin_show_about), plugin);
           g_object_set_qdata (G_OBJECT (menu), item_about, item);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+          image = gtk_image_new_from_icon_name ("gtk-about", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
           if (PANEL_HAS_FLAG (plugin->priv->flags, PLUGIN_FLAG_SHOW_ABOUT))
             gtk_widget_show (item);
 
@@ -1219,13 +1241,19 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
           /* remove */
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-          item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
+          item = gtk_image_menu_item_new_with_mnemonic (_("_Remove"));
 G_GNUC_END_IGNORE_DEPRECATIONS
           g_signal_connect_swapped (G_OBJECT (item), "activate",
               G_CALLBACK (xfce_panel_plugin_menu_remove), plugin);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           gtk_widget_show (item);
 
+          image = gtk_image_new_from_icon_name ("gtk-remove", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
+          gtk_widget_show (image);
+
           /* separator */
           item = gtk_separator_menu_item_new ();
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1299,22 +1327,34 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
       /* help item */
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      item = gtk_image_menu_item_new_from_stock (GTK_STOCK_HELP, NULL);
+      item = gtk_image_menu_item_new_with_mnemonic (_("_Help"));
 G_GNUC_END_IGNORE_DEPRECATIONS
       g_signal_connect_swapped (G_OBJECT (item), "activate",
           G_CALLBACK (xfce_panel_plugin_menu_panel_help), plugin);
       gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
       gtk_widget_show (item);
 
+      image = gtk_image_new_from_icon_name ("gtk-help", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
+      gtk_widget_show (image);
+
       /* about item */
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
+      item = gtk_image_menu_item_new_with_mnemonic (_("About"));
 G_GNUC_END_IGNORE_DEPRECATIONS
       g_signal_connect_swapped (G_OBJECT (item), "activate",
           G_CALLBACK (xfce_panel_plugin_menu_panel_about), plugin);
       gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
       gtk_widget_show (item);
 
+      image = gtk_image_new_from_icon_name ("help-about", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
+      gtk_widget_show (image);
+
       /* set panel menu */
       plugin->priv->menu = GTK_MENU (menu);
     }
