@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2017 Ali Abdallah <ali@xfce.org>
  * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,7 +38,7 @@
 #endif
 
 #include <glib.h>
-#include <dbus/dbus-glib.h>
+#include <gio/gio.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <libwnck/libwnck.h>
@@ -50,7 +51,7 @@
 #include <panel/panel-dbus-client.h>
 #include <panel/panel-preferences-dialog.h>
 
-
+static PanelApplication *application = NULL;
 
 static gint       opt_preferences = -1;
 static gint       opt_add_items = -1;
@@ -198,11 +199,32 @@ panel_debug_notify_proxy (void)
 
 
 
+static void
+panel_dbus_name_lost (GDBusConnection *connection,
+                      const gchar     *name,
+                      gpointer         user_data)
+{
+  g_critical (_("Name %s lost on the message dbus, exiting."), name);
+  gtk_main_quit ();
+}
+
+
+
+static void
+panel_dbus_name_acquired (GDBusConnection *connection,
+                          const gchar     *name,
+                          gpointer         user_data)
+{
+  application = panel_application_get ();
+  panel_application_load (application, opt_disable_wm_check);
+}
+
+
+
 gint
 main (gint argc, gchar **argv)
 {
   GOptionContext   *context;
-  PanelApplication *application;
   GError           *error = NULL;
   PanelDBusService *dbus_service;
   gboolean          succeed = FALSE;
@@ -268,7 +290,7 @@ main (gint argc, gchar **argv)
   else if (opt_preferences >= 0)
     {
       /* send a signal to the running instance to show the preferences dialog */
-      succeed = panel_dbus_client_display_preferences_dialog (opt_preferences, opt_socket_id, &error);
+      succeed = panel_dbus_client_display_preferences_dialog (opt_preferences, &error);
       goto dbus_return;
     }
   else if (opt_add_items >= 0)
@@ -311,9 +333,17 @@ main (gint argc, gchar **argv)
 
   launch_panel:
 
+  g_bus_own_name (G_BUS_TYPE_SESSION,
+                  PANEL_DBUS_NAME,
+                  G_BUS_NAME_OWNER_FLAGS_NONE,
+                  NULL,
+                  panel_dbus_name_acquired,
+                  panel_dbus_name_lost,
+                  NULL, NULL);
+
   /* start dbus service */
   dbus_service = panel_dbus_service_get ();
-  if (!panel_dbus_service_is_owner (dbus_service))
+  if (!panel_dbus_service_is_exported (dbus_service))
     {
       /* quit without error if an instance is running */
       succeed = TRUE;
@@ -342,9 +372,6 @@ main (gint argc, gchar **argv)
   /* set EWMH source indication */
   wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
 
-  application = panel_application_get ();
-  panel_application_load (application, opt_disable_wm_check);
-
   /* open dialog if we started from launch_panel */
   if (opt_preferences >= 0)
     panel_preferences_dialog_show_from_id (opt_preferences, opt_socket_id);
@@ -354,10 +381,15 @@ main (gint argc, gchar **argv)
   /* make sure there are no incomming events when we close */
   g_object_unref (G_OBJECT (dbus_service));
 
-  /* destroy all the opened dialogs */
-  panel_application_destroy_dialogs (application);
+  /* Application is set on name acquired, otherwise it is NULL */
+  if (application)
+    {
+      /* destroy all the opened dialogs */
+      panel_application_destroy_dialogs (application);
+
+      g_object_unref (G_OBJECT (application));
+    }
 
-  g_object_unref (G_OBJECT (application));
   g_object_unref (G_OBJECT (sm_client));
 
   if (panel_dbus_service_get_restart ())
@@ -393,7 +425,7 @@ dbus_return:
         error_msg = _("Failed to send D-Bus message");
 
       /* show understandable message for this common error */
-      if (error->code == DBUS_GERROR_NAME_HAS_NO_OWNER)
+      if (error->code == G_DBUS_ERROR_NAME_HAS_NO_OWNER)
         {
           /* normally start the panel */
           if (opt_preferences >= 0 || opt_restart)
