@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
+ * Copyright (c) 2017      Ali Abdallah  <ali@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
 #include <stdlib.h>
 #endif
 
-#include <dbus/dbus-glib.h>
+#include <gio/gio.h>
 #include <libxfce4util/libxfce4util.h>
 #include <common/panel-private.h>
 #include <common/panel-dbus.h>
@@ -34,8 +35,7 @@
 #include <panel/panel-dbus-client.h>
 #include <panel/panel-dbus-service.h>
 
-#include <panel/panel-dbus-client-infos.h>
-
+#include <panel/panel-gdbus-exported-service.h>
 
 
 enum
@@ -49,42 +49,36 @@ enum
 
 
 
-static DBusGProxy *
+static XfcePanelExportedService *
 panel_dbus_client_get_proxy (GError **error)
 {
-  DBusGConnection *dbus_connection;
-
-  /* return null if no connection is found */
-  dbus_connection = dbus_g_bus_get (DBUS_BUS_SESSION, error);
-  if (G_UNLIKELY (dbus_connection == NULL))
-    return NULL;
-
-  return dbus_g_proxy_new_for_name_owner (dbus_connection,
-                                          PANEL_DBUS_NAME,
-                                          PANEL_DBUS_PATH,
-                                          PANEL_DBUS_INTERFACE,
-                                          error);
+  return xfce_panel_exported_service_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                             G_DBUS_PROXY_FLAGS_NONE,
+                                                             PANEL_DBUS_NAME,
+                                                             PANEL_DBUS_PATH,
+                                                             NULL,
+                                                             error);
 }
 
 
 
 gboolean
 panel_dbus_client_display_preferences_dialog (guint         active,
-                                              const gchar  *socket_id,
                                               GError      **error)
 {
-  gboolean    result;
-  DBusGProxy *dbus_proxy;
+  XfcePanelExportedService *dbus_proxy;
+  gboolean                  result;
 
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   dbus_proxy = panel_dbus_client_get_proxy (error);
-  if (G_LIKELY (dbus_proxy == NULL))
+  if (G_UNLIKELY (dbus_proxy == NULL))
     return FALSE;
 
-  result = _panel_dbus_client_display_preferences_dialog (dbus_proxy,
-                                                          active, socket_id,
-                                                          error);
+  result = xfce_panel_exported_service_call_display_preferences_dialog_sync (dbus_proxy,
+                                                                             active,
+                                                                             NULL,
+                                                                             error);
 
   g_object_unref (G_OBJECT (dbus_proxy));
 
@@ -97,8 +91,9 @@ gboolean
 panel_dbus_client_display_items_dialog (guint    active,
                                         GError **error)
 {
-  gboolean    result;
-  DBusGProxy *dbus_proxy;
+  XfcePanelExportedService *dbus_proxy;
+  gboolean                  result;
+
 
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -106,8 +101,10 @@ panel_dbus_client_display_items_dialog (guint    active,
   if (G_UNLIKELY (dbus_proxy == NULL))
     return FALSE;
 
-  result = _panel_dbus_client_display_items_dialog (dbus_proxy, active,
-                                                    error);
+  result = xfce_panel_exported_service_call_display_items_dialog_sync (dbus_proxy,
+                                                                       active,
+                                                                       NULL,
+                                                                       error);
 
   g_object_unref (G_OBJECT (dbus_proxy));
 
@@ -119,8 +116,8 @@ panel_dbus_client_display_items_dialog (guint    active,
 gboolean
 panel_dbus_client_save (GError **error)
 {
-  DBusGProxy *dbus_proxy;
-  gboolean    result;
+  XfcePanelExportedService *dbus_proxy;
+  gboolean                  result;
 
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -128,7 +125,7 @@ panel_dbus_client_save (GError **error)
   if (G_UNLIKELY (dbus_proxy == NULL))
     return FALSE;
 
-  result = _panel_dbus_client_save (dbus_proxy, error);
+  result = xfce_panel_exported_service_call_save_sync (dbus_proxy, NULL, error);
 
   g_object_unref (G_OBJECT (dbus_proxy));
 
@@ -142,8 +139,10 @@ panel_dbus_client_add_new_item (const gchar  *plugin_name,
                                 gchar       **arguments,
                                 GError      **error)
 {
-  DBusGProxy *dbus_proxy;
-  gboolean    result;
+
+  XfcePanelExportedService *dbus_proxy;
+  gboolean                  result;
+  const gchar              *empty_arguments[] = {"", NULL};
 
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -151,9 +150,16 @@ panel_dbus_client_add_new_item (const gchar  *plugin_name,
   if (G_UNLIKELY (dbus_proxy == NULL))
     return FALSE;
 
-  result = _panel_dbus_client_add_new_item (dbus_proxy, plugin_name,
-                                            (const gchar **) arguments,
-                                            error);
+  if (arguments != NULL && *arguments != NULL)
+    result = xfce_panel_exported_service_call_add_new_item_sync (dbus_proxy, plugin_name,
+                                                                 (const gchar **) arguments,
+                                                                 NULL,
+                                                                 error);
+  else
+    result = xfce_panel_exported_service_call_add_new_item_sync (dbus_proxy, plugin_name,
+                                                                 empty_arguments,
+                                                                 NULL,
+                                                                 error);
 
   g_object_unref (G_OBJECT (dbus_proxy));
 
@@ -186,12 +192,12 @@ panel_dbus_client_plugin_event (const gchar  *plugin_event,
                                 gboolean     *return_succeed,
                                 GError      **error)
 {
-  DBusGProxy  *dbus_proxy;
-  gboolean     result = FALSE;
-  gchar      **tokens;
-  GType        type;
-  GValue       value = { 0, };
-  guint        n_tokens;
+  XfcePanelExportedService  *dbus_proxy;
+  gchar                    **tokens;
+  GType                      type;
+  guint                      n_tokens;
+  gboolean                   result = FALSE;
+  GVariant                  *variant = NULL;
 
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -214,26 +220,23 @@ panel_dbus_client_plugin_event (const gchar  *plugin_event,
   else if (n_tokens == 2)
     {
       /* set noop value, recognized by the dbus service as %NULL value */
-      g_value_init (&value, G_TYPE_UCHAR);
-      g_value_set_uchar (&value, '\0');
+      variant = g_variant_new_byte ('\0');
     }
   else if (n_tokens == N_TOKENS)
     {
       type = panel_dbus_client_gtype_from_string (tokens[TYPE]);
       if (G_LIKELY (type != G_TYPE_NONE))
         {
-          g_value_init (&value, type);
-
           if (type == G_TYPE_BOOLEAN)
-            g_value_set_boolean (&value, strcmp (tokens[VALUE], "true") == 0);
+            variant = g_variant_new_boolean (strcmp (tokens[VALUE], "true") == 0);
           else if (type == G_TYPE_DOUBLE)
-            g_value_set_double (&value, g_ascii_strtod (tokens[VALUE], NULL));
+            variant = g_variant_new_double (g_ascii_strtod (tokens[VALUE], NULL));
           else if (type == G_TYPE_INT)
-            g_value_set_int (&value, strtol (tokens[VALUE], NULL, 0));
+            variant = g_variant_new_int64 (g_ascii_strtoll (tokens[VALUE], NULL, 0));
           else if (type == G_TYPE_STRING)
-            g_value_set_static_string (&value, tokens[VALUE]);
+            variant = g_variant_new_string (tokens[VALUE]);
           else if (type == G_TYPE_UINT)
-            g_value_set_uint (&value, strtol (tokens[VALUE], NULL, 0));
+            variant = g_variant_new_uint64 (g_ascii_strtoll (tokens[VALUE], NULL, 0));
           else
             panel_assert_not_reached ();
         }
@@ -253,15 +256,14 @@ panel_dbus_client_plugin_event (const gchar  *plugin_event,
     }
 
   /* send value over dbus */
-  panel_return_val_if_fail (G_IS_VALUE (&value), FALSE);
-  result = _panel_dbus_client_plugin_event (dbus_proxy,
-                                            tokens[PLUGIN_NAME],
-                                            tokens[NAME],
-                                            &value,
-                                            return_succeed,
-                                            error);
-  g_value_unset (&value);
-
+  panel_return_val_if_fail (variant != NULL, FALSE);
+  result = xfce_panel_exported_service_call_plugin_event_sync (dbus_proxy,
+                                                               tokens[PLUGIN_NAME],
+                                                               tokens[NAME],
+                                                               g_variant_new_variant(variant),
+                                                               return_succeed,
+                                                               NULL,
+                                                               error);
 out:
   g_strfreev (tokens);
   g_object_unref (G_OBJECT (dbus_proxy));
@@ -275,8 +277,8 @@ gboolean
 panel_dbus_client_terminate (gboolean   restart,
                              GError   **error)
 {
-  DBusGProxy *dbus_proxy;
-  gboolean    result;
+  XfcePanelExportedService *dbus_proxy;
+  gboolean                  result;
 
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -284,7 +286,7 @@ panel_dbus_client_terminate (gboolean   restart,
   if (G_UNLIKELY (dbus_proxy == NULL))
     return FALSE;
 
-  result = _panel_dbus_client_terminate (dbus_proxy, restart, error);
+  result = xfce_panel_exported_service_call_terminate_sync (dbus_proxy, restart, NULL, error);
 
   g_object_unref (G_OBJECT (dbus_proxy));
 
