@@ -21,7 +21,6 @@
 #endif
 
 #include <gmodule.h>
-#include <exo/exo.h>
 #include <glib/gstdio.h>
 #include <libxfce4util/libxfce4util.h>
 
@@ -101,6 +100,9 @@ struct _PanelModule
 
   /* for gobject plugins */
   GType                plugin_type;
+
+  /* for wrapper plugins */
+  gchar               *api;
 };
 
 
@@ -145,6 +147,7 @@ panel_module_init (PanelModule *module)
   module->library = NULL;
   module->construct_func = NULL;
   module->plugin_type = G_TYPE_NONE;
+  module->api = g_strdup (LIBXFCE4PANEL_VERSION_API);
 }
 
 
@@ -171,6 +174,7 @@ panel_module_finalize (GObject *object)
   g_free (module->display_name);
   g_free (module->comment);
   g_free (module->icon_name);
+  g_free (module->api);
 
   (*G_OBJECT_CLASS (panel_module_parent_class)->finalize) (object);
 }
@@ -215,6 +219,8 @@ panel_module_load (GTypeModule *type_module)
 
       /* from now on, run this plugin in a wrapper */
       module->mode = WRAPPER;
+      g_free (module->api);
+      module->api = g_strdup (LIBXFCE4PANEL_VERSION_API);
 
       return FALSE;
     }
@@ -304,8 +310,8 @@ panel_module_new_from_desktop_file (const gchar *filename,
   const gchar *module_unique;
   gboolean     found;
 
-  panel_return_val_if_fail (!exo_str_is_empty (filename), NULL);
-  panel_return_val_if_fail (!exo_str_is_empty (name), NULL);
+  panel_return_val_if_fail (!panel_str_is_empty (filename), NULL);
+  panel_return_val_if_fail (!panel_str_is_empty (name), NULL);
 
   rc = xfce_rc_simple_open (filename, TRUE);
   if (G_UNLIKELY (rc == NULL))
@@ -360,7 +366,12 @@ panel_module_new_from_desktop_file (const gchar *filename,
           /* run mode of the module, by default everything runs in
            * the wrapper, unless defined otherwise */
           if (force_external || !xfce_rc_read_bool_entry (rc, "X-XFCE-Internal", FALSE))
-            module->mode = WRAPPER;
+            {
+              module->mode = WRAPPER;
+              g_free (module->api);
+              //module->api = g_strdup (xfce_rc_read_entry (rc, "X-XFCE-API", LIBXFCE4PANEL_VERSION_API));
+              module->api = g_strdup (xfce_rc_read_entry (rc, "X-XFCE-API", "1.0"));
+            }
           else
             module->mode = INTERNAL;
         }
@@ -576,6 +587,17 @@ panel_module_get_icon_name (PanelModule *module)
 
 
 
+const gchar *
+panel_module_get_api (PanelModule *module)
+{
+  panel_return_val_if_fail (PANEL_IS_MODULE (module), NULL);
+  panel_return_val_if_fail (G_IS_TYPE_MODULE (module), NULL);
+
+  return module->api;
+}
+
+
+
 PanelModule *
 panel_module_get_from_plugin_provider (XfcePanelPluginProvider *provider)
 {
