@@ -1,6 +1,5 @@
 /*
  * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
- * Copyright (C) 2014 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +24,6 @@
 #include <stdlib.h>
 #endif
 
-#include <exo/exo.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 
@@ -56,7 +54,8 @@ static void                     panel_preferences_dialog_response
 static void                     panel_preferences_dialog_bindings_unbind        (PanelPreferencesDialog *dialog);
 static void                     panel_preferences_dialog_bindings_add           (PanelPreferencesDialog *dialog,
                                                                                  const gchar            *property1,
-                                                                                 const gchar            *property2);
+                                                                                 const gchar            *property2,
+                                                                                 GBindingFlags           flags);
 static void                     panel_preferences_dialog_bindings_update        (PanelPreferencesDialog *dialog);
 static void                     panel_preferences_dialog_output_changed         (GtkComboBox            *combobox,
                                                                                  PanelPreferencesDialog *dialog);
@@ -131,7 +130,7 @@ struct _PanelPreferencesDialog
   /* currently selected window in the selector */
   PanelWindow      *active;
 
-  /* ExoMutualBinding's between dialog <-> window */
+  /* GBinding's between dialog <-> window */
   GSList           *bindings;
 
   /* store for the items list */
@@ -230,8 +229,9 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
 
   info = gtk_builder_get_object (GTK_BUILDER (dialog), "composited-info");
   panel_return_if_fail (G_IS_OBJECT (info));
-  exo_binding_new_with_negation (G_OBJECT (object), "sensitive",
-                                 G_OBJECT (info), "visible");
+  g_object_bind_property (G_OBJECT (object), "sensitive",
+                          G_OBJECT (info), "visible",
+                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-image");
   panel_return_if_fail (GTK_IS_FILE_CHOOSER_BUTTON (object));
@@ -378,7 +378,7 @@ panel_preferences_dialog_bindings_unbind (PanelPreferencesDialog *dialog)
     {
       /* remove all bindings */
       for (li = dialog->bindings; li != NULL; li = li->next)
-        exo_mutual_binding_unbind (li->data);
+        g_object_unref (G_OBJECT (li->data));
 
       g_slist_free (dialog->bindings);
       dialog->bindings = NULL;
@@ -402,17 +402,19 @@ panel_preferences_dialog_bindings_unbind (PanelPreferencesDialog *dialog)
 static void
 panel_preferences_dialog_bindings_add (PanelPreferencesDialog *dialog,
                                        const gchar            *property1,
-                                       const gchar            *property2)
+                                       const gchar            *property2,
+                                       GBindingFlags           flags)
 {
-  ExoMutualBinding *binding;
-  GObject          *object;
+  GBinding *binding;
+  GObject  *object;
 
   /* get the object from the builder */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), property1);
   panel_return_if_fail (G_IS_OBJECT (object));
 
   /* create the binding and prepend to the list */
-  binding = exo_mutual_binding_new (G_OBJECT (dialog->active), property1, object, property2);
+  binding = g_object_bind_property (G_OBJECT (dialog->active), property1, object, property2,
+                                    flags ? flags : G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
   dialog->bindings = g_slist_prepend (dialog->bindings, binding);
 }
 
@@ -440,31 +442,31 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
     return;
 
   /* hook up the bindings */
-  panel_preferences_dialog_bindings_add (dialog, "mode", "active");
-  panel_preferences_dialog_bindings_add (dialog, "span-monitors", "active");
-  panel_preferences_dialog_bindings_add (dialog, "position-locked", "active");
-  panel_preferences_dialog_bindings_add (dialog, "autohide-behavior", "active");
-  panel_preferences_dialog_bindings_add (dialog, "disable-struts", "active");
-  panel_preferences_dialog_bindings_add (dialog, "size", "value");
-  panel_preferences_dialog_bindings_add (dialog, "nrows", "value");
-  panel_preferences_dialog_bindings_add (dialog, "length", "value");
-  panel_preferences_dialog_bindings_add (dialog, "length-adjust", "active");
-  panel_preferences_dialog_bindings_add (dialog, "background-alpha", "value");
-  panel_preferences_dialog_bindings_add (dialog, "enter-opacity", "value");
-  panel_preferences_dialog_bindings_add (dialog, "leave-opacity", "value");
-  panel_preferences_dialog_bindings_add (dialog, "composited", "sensitive");
-  panel_preferences_dialog_bindings_add (dialog, "background-style", "active");
-  panel_preferences_dialog_bindings_add (dialog, "background-color", "color");
+  panel_preferences_dialog_bindings_add (dialog, "mode", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "span-monitors", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "position-locked", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "autohide-behavior", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "disable-struts", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "size", "value", 0);
+  panel_preferences_dialog_bindings_add (dialog, "nrows", "value", 0);
+  panel_preferences_dialog_bindings_add (dialog, "length", "value", 0);
+  panel_preferences_dialog_bindings_add (dialog, "length-adjust", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "enter-opacity", "value", 0);
+  panel_preferences_dialog_bindings_add (dialog, "leave-opacity", "value", 0);
+  panel_preferences_dialog_bindings_add (dialog, "composited", "sensitive", G_BINDING_SYNC_CREATE);
+  panel_preferences_dialog_bindings_add (dialog, "background-style", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "background-rgba", "rgba", 0);
 
   /* watch image changes from the panel */
   dialog->bg_image_notify_handler_id = g_signal_connect_swapped (G_OBJECT (dialog->active),
       "notify::background-image", G_CALLBACK (panel_preferences_dialog_bg_image_notified), dialog);
   panel_preferences_dialog_bg_image_notified (dialog);
 
+
   /* get run mode of the driver (multiple screens or randr) */
   screen = gtk_widget_get_screen (GTK_WIDGET (dialog->active));
   display = gtk_widget_get_display (GTK_WIDGET (dialog->active));
-  n_screens = gdk_display_get_n_screens (display);
+  n_screens = 1;
   n_monitors = 1;
   if (G_LIKELY (n_screens <= 1))
     {
@@ -485,22 +487,21 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
 
   if (n_screens > 1
       || n_monitors > 1
-      || !exo_str_is_empty (output_name))
+      || !panel_str_is_empty (output_name))
     {
       gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                          OUTPUT_NAME, "Automatic",
                                          OUTPUT_TITLE, _("Automatic"), -1);
-      if (exo_str_is_empty (output_name) ||
-          g_strcmp0 (output_name, "Automatic") == 0)
-        {
-          gtk_combo_box_set_active_iter (GTK_COMBO_BOX (object), &iter);
-          output_selected = TRUE;
-          span_monitors_sensitive = TRUE;
-        }
+      if (g_strcmp0 (output_name, "Automatic") == 0) {
+        gtk_combo_box_set_active_iter (GTK_COMBO_BOX (object), &iter);
+        output_selected = TRUE;
+        span_monitors_sensitive = TRUE;
+      }
       gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                          OUTPUT_NAME, "Primary",
                                          OUTPUT_TITLE, _("Primary"), -1);
-      if (g_strcmp0 (output_name, "Primary") == 0)
+      if (panel_str_is_empty (output_name) ||
+          g_strcmp0 (output_name, "Primary") == 0)
         {
           gtk_combo_box_set_active_iter (GTK_COMBO_BOX (object), &iter);
           output_selected = TRUE;
@@ -524,7 +525,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
                                                  OUTPUT_NAME, name,
                                                  OUTPUT_TITLE, title, -1);
 
-              if (!output_selected && exo_str_is_equal (name, output_name))
+              if (!output_selected && g_strcmp0 (name, output_name) == 0)
                 {
                   gtk_combo_box_set_active_iter  (GTK_COMBO_BOX (object), &iter);
                   output_selected = TRUE;
@@ -539,7 +540,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
           for (i = 0; i < n_monitors; i++)
             {
               name = gdk_screen_get_monitor_plug_name (screen, i);
-              if (exo_str_is_empty (name))
+              if (panel_str_is_empty (name))
                 {
                   g_free (name);
 
@@ -556,7 +557,8 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
               gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                                  OUTPUT_NAME, name,
                                                  OUTPUT_TITLE, title, -1);
-              if (!output_selected && exo_str_is_equal (name, output_name))
+              if (!output_selected
+                  && g_strcmp0 (name, output_name) == 0)
                 {
                   gtk_combo_box_set_active_iter  (GTK_COMBO_BOX (object), &iter);
                   output_selected = TRUE;
@@ -568,7 +570,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
         }
 
       /* add the output from the config if still nothing has been selected */
-      if (!output_selected && !exo_str_is_empty (output_name))
+      if (!output_selected && !panel_str_is_empty (output_name))
         {
           gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                              OUTPUT_NAME, output_name,
@@ -672,11 +674,10 @@ panel_preferences_dialog_bg_style_changed (PanelPreferencesDialog *dialog)
   panel_return_if_fail (GTK_IS_COMBO_BOX (object));
   active = gtk_combo_box_get_active (GTK_COMBO_BOX (object));
 
-  object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-alpha-box");
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-rgba");
   panel_return_if_fail (GTK_IS_WIDGET (object));
   g_object_get (G_OBJECT (dialog->active), "composited", &composited, NULL);
-  g_object_set (G_OBJECT (object), "visible", active < 2,
-                "sensitive", composited, NULL);
+  gtk_color_chooser_set_use_alpha (GTK_COLOR_CHOOSER (object), composited);
 
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-color-box");
   panel_return_if_fail (GTK_IS_WIDGET (object));
@@ -925,7 +926,7 @@ panel_preferences_dialog_panel_remove (GtkWidget              *widget,
     return;
 
   toplevel = gtk_widget_get_toplevel (widget);
-  if (xfce_dialog_confirm (GTK_WINDOW (toplevel), GTK_STOCK_REMOVE, NULL,
+  if (xfce_dialog_confirm (GTK_WINDOW (toplevel), "list-remove", _("Remove"),
           _("The panel and plugin configurations will be permanently removed"),
           _("Are you sure you want to remove panel %d?"),
           panel_window_get_id (dialog->active)))
@@ -1202,8 +1203,8 @@ panel_preferences_dialog_item_remove (GtkWidget              *button,
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (widget),
                                                 _("If you remove the item from the panel, "
                                                   "it is permanently lost."));
-      gtk_dialog_add_buttons (GTK_DIALOG (widget), GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
-                              GTK_STOCK_REMOVE, GTK_RESPONSE_YES, NULL);
+      gtk_dialog_add_buttons (GTK_DIALOG (widget), _("Cancel"), GTK_RESPONSE_NO,
+                              _("Remove"), GTK_RESPONSE_YES, NULL);
       gtk_dialog_set_default_response (GTK_DIALOG (widget), GTK_RESPONSE_NO);
 
       /* run the dialog */
@@ -1406,8 +1407,8 @@ panel_preferences_dialog_plug_deleted (GtkWidget *plug)
 
 
 static void
-panel_preferences_dialog_show_internal (PanelWindow     *active,
-                                        GdkNativeWindow  socket_window)
+panel_preferences_dialog_show_internal (PanelWindow *active,
+                                        Window       socket_window)
 {
   gint         panel_id = 0;
   GObject     *window, *combo;
@@ -1457,7 +1458,7 @@ panel_preferences_dialog_show_internal (PanelWindow     *active,
 
       /* move the vbox to the dialog */
       content_area = gtk_dialog_get_content_area (GTK_DIALOG (window));
-      gtk_widget_reparent (GTK_WIDGET (plug_child), content_area);
+      xfce_widget_reparent (GTK_WIDGET (plug_child), content_area);
       gtk_widget_show (GTK_WIDGET (plug_child));
 
       /* destroy the plug */
@@ -1494,7 +1495,7 @@ panel_preferences_dialog_show_internal (PanelWindow     *active,
       gtk_widget_show (plug);
 
       /* move the vbox in the plug */
-      gtk_widget_reparent (GTK_WIDGET (plug_child), plug);
+      xfce_widget_reparent (GTK_WIDGET (plug_child), plug);
       gtk_widget_show (GTK_WIDGET (plug_child));
     }
 }
@@ -1516,12 +1517,12 @@ panel_preferences_dialog_show_from_id (gint         panel_id,
 {
   PanelApplication *application;
   PanelWindow      *window;
-  GdkNativeWindow   socket_window = 0;
+  Window            socket_window = 0;
 
   /* x11 windows are ulong on 64 bit platforms
    * or uint32 on other platforms */
   if (socket_id != NULL)
-    socket_window = (GdkNativeWindow) strtoul (socket_id, NULL, 0);
+    socket_window = (Window) strtoul (socket_id, NULL, 0);
 
   application = panel_application_get ();
   window = panel_application_get_window (application, panel_id);
