@@ -274,7 +274,7 @@ launcher_plugin_class_init (LauncherPluginClass *klass)
                                    PROP_ITEMS,
                                    g_param_spec_boxed ("items",
                                                        NULL, NULL,
-                                                       PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
+                                                       G_TYPE_PTR_ARRAY,
                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
@@ -401,6 +401,17 @@ launcher_plugin_init (LauncherPlugin *plugin)
 
 
 
+static void
+launcher_free_array_element (gpointer data)
+{
+  GValue *value = (GValue *)data;
+
+  g_value_unset (value);
+  g_free (value);
+}
+
+
+
 static void
 launcher_plugin_get_property (GObject    *object,
                               guint       prop_id,
@@ -416,7 +427,7 @@ launcher_plugin_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_ITEMS:
-      array = g_ptr_array_new ();
+      array = g_ptr_array_new_full (1, (GDestroyNotify) launcher_free_array_element);
       for (li = plugin->items; li != NULL; li = li->next)
         {
           tmp = g_new0 (GValue, 1);
@@ -431,7 +442,7 @@ launcher_plugin_get_property (GObject    *object,
           g_ptr_array_add (array, tmp);
         }
       g_value_set_boxed (value, array);
-      xfconf_array_free (array);
+      g_ptr_array_unref (array);
       break;
 
     case PROP_DISABLE_TOOLTIPS:
@@ -999,7 +1010,7 @@ launcher_plugin_construct (XfcePanelPlugin *panel_plugin)
   const PanelProperty  properties[] =
   {
     { "show-label", G_TYPE_BOOLEAN },
-    { "items", PANEL_PROPERTIES_TYPE_VALUE_ARRAY },
+    { "items", G_TYPE_PTR_ARRAY },
     { "disable-tooltips", G_TYPE_BOOLEAN },
     { "move-first", G_TYPE_BOOLEAN },
     { "arrow-position", G_TYPE_UINT },
